import { __decorate, __metadata } from "tslib";
import { MdcComponent } from '@aurelia2-mdc-web/base';
import { MDCMenuSurfaceFoundation, cssClasses, Corner, strings } from '@material/menu-surface';
import { inject, customAttribute, bindable } from 'aurelia';
import { getCorrectPropertyName } from '@material/animation/util';
import { INode } from 'aurelia';
strings.OPENED_EVENT = strings.OPENED_EVENT.toLowerCase();
strings.CLOSED_EVENT = strings.CLOSED_EVENT.toLowerCase();
let MdcMenuSurface = class MdcMenuSurface extends MdcComponent {
    originalParent;
    previousFocus;
    anchor;
    /** Sets the foundation to use page offsets for an positioning when the menu is hoisted to the body. */
    hoistToBody;
    fixed;
    async fixedChanged() {
        if (this.fixed) {
            this.root.classList.add(cssClasses.FIXED);
        }
        else {
            this.root.classList.remove(cssClasses.FIXED);
        }
        await this.initialised;
        this.foundation?.setFixedPosition(this.fixed);
    }
    anchorCorner;
    async anchorCornerChanged() {
        await this.initialised;
        this.foundation?.setAnchorCorner(Corner[this.anchorCorner]);
    }
    anchorMargin;
    async anchorMarginChanged(margin) {
        await this.initialised;
        this.foundation?.setAnchorMargin(margin);
    }
    quickOpen;
    async quickOpenChanged(quickOpen) {
        await this.initialised;
        this.foundation?.setQuickOpen(quickOpen);
    }
    fullWidth;
    fullWidthChanged() {
        if (this.fullWidth) {
            this.root.classList.add('mdc-menu-surface--fullwidth');
        }
        else {
            this.root.classList.remove('mdc-menu-surface--fullwidth');
        }
    }
    stayOpen;
    maxHeight;
    async maxHeightChanged() {
        await this.initialised;
        this.foundation?.setMaxHeight(this.maxHeight);
    }
    horizontallyCenteredOnViewport;
    async horizontallyCenteredOnViewportChanged() {
        await this.initialised;
        this.foundation?.setIsHorizontallyCenteredOnViewport(this.horizontallyCenteredOnViewport);
    }
    openBottomBias;
    async openBottomBiasChanged() {
        await this.initialised;
        this.foundation?.setOpenBottomBias(this.openBottomBias);
    }
    get open() {
        return this.foundation?.isOpen() ?? false;
    }
    set open(value) {
        if (value) {
            this.foundation?.open();
        }
        else {
            this.foundation?.close();
        }
    }
    handleKeydown(evt) {
        this.foundation?.handleKeydown(evt);
    }
    handleBodyClick(evt) {
        if (!this.stayOpen) {
            this.foundation?.handleBodyClick(evt);
        }
    }
    // eslint-disable-next-line @typescript-eslint/require-await
    async initialise() {
        this.root.classList.add(cssClasses.ROOT);
    }
    handleEvent(evt) {
        switch (evt.currentTarget) {
            case this.root:
                switch (evt.type) {
                    case 'keydown':
                        this.handleKeydown(evt);
                        break;
                }
                break;
            case document.body:
                switch (evt.type) {
                    case 'click':
                        this.handleBodyClick(evt);
                        break;
                }
                break;
        }
    }
    registerBodyClickListener() {
        // this is different to native implementation
        // refer to https://github.com/material-components/material-components-web/issues/6188
        // capture so that no race between handleBodyClick and quickOpen when
        // menusurface opened on button click which registers this listener
        document.body.addEventListener('click', this);
    }
    deregisterBodyClickListener() {
        document.body.removeEventListener('click', this);
    }
    initialSyncWithDOM() {
        const parentEl = this.root.parentElement;
        if (!this.anchor) {
            this.anchor = parentEl?.classList.contains(cssClasses.ANCHOR) ? parentEl : null;
        }
        if (this.hoistToBody) {
            this.originalParent = this.root.parentElement;
            if (this.originalParent) {
                document.body.appendChild(this.originalParent.removeChild(this.root));
                this.foundation?.setIsHoisted(true);
            }
        }
        this.foundation?.setFixedPosition(this.fixed);
        if (this.anchorCorner) {
            this.foundation?.setAnchorCorner(Corner[this.anchorCorner]);
        }
        if (this.anchorMargin) {
            this.foundation?.setAnchorMargin(this.anchorMargin);
        }
        if (this.maxHeight) {
            this.foundation?.setMaxHeight(this.maxHeight);
        }
        if (this.horizontallyCenteredOnViewport !== undefined) {
            this.foundation?.setIsHorizontallyCenteredOnViewport(this.horizontallyCenteredOnViewport);
        }
        this.listen('keydown', this);
    }
    destroy() {
        this.unlisten('keydown', this);
        if (this.originalParent) {
            this.originalParent.appendChild(document.body.removeChild(this.root));
        }
        super.destroy();
    }
    close(skipRestoreFocus = false) {
        this.foundation?.close(skipRestoreFocus);
    }
    /** Sets the absolute x/y position to position based on. Requires the menu to be hoisted. */
    setAbsolutePosition(x, y) {
        this.foundation?.setAbsolutePosition(x, y);
        this.hoistToBody = true;
    }
    /**
     * @param corner Default anchor corner alignment of top-left surface corner.
     */
    setAnchorCorner(corner) {
        this.foundation?.setAnchorCorner(corner);
    }
    getDefaultFoundation() {
        // DO NOT INLINE this variable. For backward compatibility, foundations take a Partial<MDCFooAdapter>.
        // To ensure we don't accidentally omit any methods, we need a separate, strongly typed adapter variable.
        const adapter = {
            addClass: (className) => this.root.classList.add(className),
            removeClass: (className) => this.root.classList.remove(className),
            hasClass: (className) => this.root.classList.contains(className),
            hasAnchor: () => !!this.anchor,
            notifyClose: () => {
                this.emit(MDCMenuSurfaceFoundation.strings.CLOSED_EVENT, {});
                this.deregisterBodyClickListener();
            },
            notifyClosing: () => {
                this.emit(MDCMenuSurfaceFoundation.strings.CLOSING_EVENT, {});
            },
            notifyOpen: () => {
                this.emit(MDCMenuSurfaceFoundation.strings.OPENED_EVENT, {});
                this.registerBodyClickListener();
            },
            notifyOpening: () => {
                this.emit(MDCMenuSurfaceFoundation.strings.OPENING_EVENT, {});
            },
            isElementInContainer: (el) => this.root.contains(el),
            isRtl: () => getComputedStyle(this.root).getPropertyValue('direction') === 'rtl',
            setTransformOrigin: (origin) => {
                const propertyName = `${getCorrectPropertyName(window, 'transform')}-origin`;
                this.root.style.setProperty(propertyName, origin);
            },
            isFocused: () => document.activeElement === this.root,
            saveFocus: () => {
                this.previousFocus = document.activeElement;
            },
            restoreFocus: () => {
                if (this.root.contains(document.activeElement)) {
                    if (this.previousFocus?.focus) {
                        this.previousFocus.focus();
                    }
                }
            },
            getInnerDimensions: () => {
                const firstChild = this.root.firstElementChild;
                // workaround https://github.com/material-components/material-components-web/issues/6709
                // by explicitly displaying vertical scroll if a list does not fit
                this.root.style.overflowY = this.root.offsetHeight < firstChild.offsetHeight ? 'scroll' : '';
                return {
                    width: this.root.offsetWidth,
                    height: this.root.offsetHeight
                };
            },
            getAnchorDimensions: () => this.anchor ? this.anchor.getBoundingClientRect() : null,
            getWindowDimensions: () => {
                return { width: window.innerWidth, height: window.innerHeight };
            },
            getBodyDimensions: () => {
                return { width: document.body.clientWidth, height: document.body.clientHeight };
            },
            getWindowScroll: () => {
                return { x: window.pageXOffset, y: window.pageYOffset };
            },
            setPosition: (position) => {
                const rootHTML = this.root;
                rootHTML.style.left = 'left' in position ? `${position.left}px` : '';
                rootHTML.style.right = 'right' in position ? `${position.right}px` : '';
                rootHTML.style.top = 'top' in position ? `${position.top}px` : '';
                rootHTML.style.bottom = 'bottom' in position ? `${position.bottom}px` : '';
            },
            setMaxHeight: (height) => {
                (this.root).style.maxHeight = height;
            },
        };
        return new MDCMenuSurfaceFoundation(adapter);
    }
};
__decorate([
    bindable,
    __metadata("design:type", Object)
], MdcMenuSurface.prototype, "anchor", void 0);
__decorate([
    bindable({ mode: 1 /* BindingMode.oneTime */ }),
    __metadata("design:type", Boolean)
], MdcMenuSurface.prototype, "hoistToBody", void 0);
__decorate([
    bindable,
    __metadata("design:type", Boolean)
], MdcMenuSurface.prototype, "fixed", void 0);
__decorate([
    bindable,
    __metadata("design:type", Object)
], MdcMenuSurface.prototype, "anchorCorner", void 0);
__decorate([
    bindable,
    __metadata("design:type", Object)
], MdcMenuSurface.prototype, "anchorMargin", void 0);
__decorate([
    bindable,
    __metadata("design:type", Boolean)
], MdcMenuSurface.prototype, "quickOpen", void 0);
__decorate([
    bindable,
    __metadata("design:type", Boolean)
], MdcMenuSurface.prototype, "fullWidth", void 0);
__decorate([
    bindable,
    __metadata("design:type", Boolean)
], MdcMenuSurface.prototype, "stayOpen", void 0);
__decorate([
    bindable,
    __metadata("design:type", Number)
], MdcMenuSurface.prototype, "maxHeight", void 0);
__decorate([
    bindable,
    __metadata("design:type", Boolean)
], MdcMenuSurface.prototype, "horizontallyCenteredOnViewport", void 0);
__decorate([
    bindable,
    __metadata("design:type", Number)
], MdcMenuSurface.prototype, "openBottomBias", void 0);
MdcMenuSurface = __decorate([
    inject(INode),
    customAttribute('mdc-menu-surface')
], MdcMenuSurface);
export { MdcMenuSurface };
//# sourceMappingURL=mdc-menu-surface.js.map